@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2004, 2005, 2006,
-   2007, 2008 Free Software Foundation, Inc.
+   2007, 2008, 2009 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -49,31 +49,23 @@ mailbox_folder_create (mu_mailbox_t mbox, const char *name,
 		       mu_record_t record)
 {
   int rc;
-  char *fname;
-
-  if ((rc = mu_url_aget_path (mbox->url, &fname)))
+  mu_url_t url;
+  
+  if ((rc = mu_url_uplevel (mbox->url, &url)))
     {
       if (rc == MU_ERR_NOENT)
 	{
-	  fname = strdup (mu_url_to_string (mbox->url));
-	  if (!fname)
-	    return ENOMEM;
+	  rc = mu_url_dup (mbox->url, &url);
+	  if (rc)
+	    return rc;
 	}
       else
 	return rc;
     }
 
-  if (mu_url_is_scheme (mbox->url, "file")
-      || mu_url_is_scheme (mbox->url, "mbox")
-      || mu_url_is_scheme (mbox->url, "mh")
-      || mu_url_is_scheme (mbox->url, "maildir"))
-    {
-      char *p = strrchr (fname, '/'); /* FIXME: Is this always appropriate? */
-      if (p)
-	*p = 0;
-    }
-  rc = mu_folder_create_from_record (&mbox->folder, fname, record);
-  free (fname);
+  rc = mu_folder_create_from_record (&mbox->folder, url, record);
+  if (rc)
+    mu_url_destroy (&url);
   return rc;
 }
 
@@ -109,20 +101,12 @@ _create_mailbox0 (mu_mailbox_t *pmbox, mu_url_t url, const char *name)
 	      return status;
 	    }
 
-	  mu_record_get_url (record, &u_init);
-	  if (u_init && (status = u_init (url)) != 0)
-	    {
-	      mu_mailbox_destroy (&mbox);
-	      return status;
-	    }
-
 	  /* Make sure scheme contains actual mailbox scheme */
 	  /* FIXME: It is appropriate not for all record types.  For now we
 	     assume that if the record scheme ends with a plus sign, this
 	     should not be done.  Probably it requires some flag in struct
 	     _mu_record? */
-	  if (record->scheme[strlen(record->scheme)-1] != '+' 
-	      && strcmp (url->scheme, record->scheme))
+	  if (strcmp (url->scheme, record->scheme))
 	    {
 	      char *p = strdup (record->scheme);
 	      if (!p)
@@ -134,6 +118,13 @@ _create_mailbox0 (mu_mailbox_t *pmbox, mu_url_t url, const char *name)
 	      url->scheme = p;
 	    }
 
+	  mu_record_get_url (record, &u_init);
+	  if (u_init && (status = u_init (url)) != 0)
+	    {
+	      mu_mailbox_destroy (&mbox);
+	      return status;
+	    }
+
 	  mbox->url = url;
 
 	  /* Create the folder before initializing the concrete mailbox.
@@ -296,7 +287,7 @@ mu_mailbox_flush (mu_mailbox_t mbox, int expunge)
 {
   size_t i, total = 0;
   int status = 0;
-
+  
   if (!mbox)
     return EINVAL;
   if (!(mbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND)))
@@ -689,4 +680,67 @@ mu_mailbox_unlock (mu_mailbox_t mbox)
   mu_mailbox_get_locker (mbox, &lock);
   return mu_locker_unlock (lock);
 }
-  
+
+static void
+free_uidl (void *item)
+{
+  free (item);
+}
+
+int
+mu_mailbox_get_uidls (mu_mailbox_t mbox, mu_list_t *plist)
+{
+  mu_list_t list;
+  int status;
+
+  if (mbox == NULL)
+    return MU_ERR_MBX_NULL;
+  if (plist == NULL)
+    return EINVAL;
+  status = mu_list_create (&list);
+  if (status)
+    return status;
+  mu_list_set_destroy_item (list, free_uidl);
+  if (mbox->_get_uidls)
+    status = mbox->_get_uidls (mbox, list);
+  else
+    {
+      size_t i, total;
+
+      status = mu_mailbox_messages_count (mbox, &total);
+      if (status)
+	return status;
+      for (i = 1; i <= total; i++)
+	{
+	  mu_message_t msg = NULL;
+	  char buf[MU_UIDL_BUFFER_SIZE];
+	  size_t n;
+	  struct mu_uidl *uidl;
+	  
+	  status = mu_mailbox_get_message (mbox, i, &msg);
+	  if (status)
+	    break;
+	  status = mu_message_get_uidl (msg, buf, sizeof (buf), &n);
+	  if (status)
+	    break;
+	  uidl = malloc (sizeof (uidl[0]));
+	  if (!uidl)
+	    {
+	      status = ENOMEM;
+	      break;
+	    }
+	  uidl->msgno = i;
+	  strncpy (uidl->uidl, strdup (buf), MU_UIDL_BUFFER_SIZE);
+	  status = mu_list_append (list, uidl);
+	  if (status)
+	    {
+	      free_uidl (uidl);
+	      break;
+	    }
+	}
+    }
+  *plist = list;
+  return status;
+}
+
+	  
