@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2007, 2009 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -101,7 +101,7 @@ mh_seq_add (const char *name, mh_msgset_t *mset, int flags)
     }
   *p = 0;
   write_sequence (name, new_value, flags & SEQ_PRIVATE);
-  if (strcasecmp (name, "cur") == 0)
+  if (mu_c_strcasecmp (name, "cur") == 0)
     current_message = strtoul (new_value, NULL, 0);
   free (new_value);
 }
